@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -28,6 +28,22 @@ class CItem extends CItemGeneral {
 	protected $tableAlias = 'i';
 	protected $sortColumns = ['itemid', 'name', 'key_', 'delay', 'history', 'trends', 'type', 'status'];
 
+	/**
+	 * Define a set of supported pre-processing rules.
+	 *
+	 * @var array
+	 *
+	 * 5.6 would allow this to be defined constant.
+	 */
+	public static $supported_preprocessing_types = [ZBX_PREPROC_REGSUB, ZBX_PREPROC_TRIM, ZBX_PREPROC_RTRIM,
+		ZBX_PREPROC_LTRIM, ZBX_PREPROC_XPATH, ZBX_PREPROC_JSONPATH, ZBX_PREPROC_MULTIPLIER, ZBX_PREPROC_DELTA_VALUE,
+		ZBX_PREPROC_DELTA_SPEED, ZBX_PREPROC_BOOL2DEC, ZBX_PREPROC_OCT2DEC, ZBX_PREPROC_HEX2DEC,
+		ZBX_PREPROC_VALIDATE_RANGE, ZBX_PREPROC_VALIDATE_REGEX, ZBX_PREPROC_VALIDATE_NOT_REGEX,
+		ZBX_PREPROC_ERROR_FIELD_JSON, ZBX_PREPROC_ERROR_FIELD_XML, ZBX_PREPROC_ERROR_FIELD_REGEX,
+		ZBX_PREPROC_THROTTLE_VALUE, ZBX_PREPROC_THROTTLE_TIMED_VALUE, ZBX_PREPROC_SCRIPT,
+		ZBX_PREPROC_PROMETHEUS_PATTERN, ZBX_PREPROC_PROMETHEUS_TO_JSON
+	];
+
 	public function __construct() {
 		parent::__construct();
 
@@ -169,7 +185,7 @@ class CItem extends CItemGeneral {
 		if (!is_null($options['interfaceids'])) {
 			zbx_value2array($options['interfaceids']);
 
-			$sqlParts['where']['interfaceid'] = dbConditionInt('i.interfaceid', $options['interfaceids']);
+			$sqlParts['where']['interfaceid'] = dbConditionId('i.interfaceid', $options['interfaceids']);
 
 			if ($options['groupCount']) {
 				$sqlParts['group']['i'] = 'i.interfaceid';
@@ -194,7 +210,7 @@ class CItem extends CItemGeneral {
 			zbx_value2array($options['proxyids']);
 
 			$sqlParts['from']['hosts'] = 'hosts h';
-			$sqlParts['where'][] = dbConditionInt('h.proxy_hostid', $options['proxyids']);
+			$sqlParts['where'][] = dbConditionId('h.proxy_hostid', $options['proxyids']);
 			$sqlParts['where'][] = 'h.hostid=i.hostid';
 
 			if ($options['groupCount']) {
@@ -309,7 +325,7 @@ class CItem extends CItemGeneral {
 
 		// group
 		if (!is_null($options['group'])) {
-			$sqlParts['from']['groups'] = 'groups g';
+			$sqlParts['from']['hstgrp'] = 'hstgrp g';
 			$sqlParts['from']['hosts_groups'] = 'hosts_groups hg';
 			$sqlParts['where']['ghg'] = 'g.groupid=hg.groupid';
 			$sqlParts['where']['hgi'] = 'hg.hostid=i.hostid';
@@ -391,6 +407,20 @@ class CItem extends CItemGeneral {
 			$result = zbx_cleanHashes($result);
 		}
 
+		// Decode ITEM_TYPE_HTTPAGENT encoded fields.
+		$json = new CJson();
+
+		foreach ($result as &$item) {
+			if (array_key_exists('query_fields', $item)) {
+				$query_fields = ($item['query_fields'] !== '') ? $json->decode($item['query_fields'], true) : [];
+				$item['query_fields'] = $json->hasError() ? [] : $query_fields;
+			}
+
+			if (array_key_exists('headers', $item)) {
+				$item['headers'] = $this->headersStringToArray($item['headers']);
+			}
+		}
+
 		return $result;
 	}
 
@@ -412,7 +442,32 @@ class CItem extends CItemGeneral {
 			unset($item['itemid']);
 		}
 		unset($item);
-		$this->validateDependentItems($items, API::Item());
+
+		$this->validateDependentItems($items);
+
+		$json = new CJson();
+
+		foreach ($items as &$item) {
+			if ($item['type'] == ITEM_TYPE_HTTPAGENT) {
+				if (array_key_exists('query_fields', $item)) {
+					$item['query_fields'] = $item['query_fields'] ? $json->encode($item['query_fields']) : '';
+				}
+
+				if (array_key_exists('headers', $item)) {
+					$item['headers'] = $this->headersArrayToString($item['headers']);
+				}
+
+				if (array_key_exists('request_method', $item) && $item['request_method'] == HTTPCHECK_REQUEST_HEAD
+						&& !array_key_exists('retrieve_mode', $item)) {
+					$item['retrieve_mode'] = HTTPTEST_STEP_RETRIEVE_MODE_HEADERS;
+				}
+			}
+			else {
+				$item['query_fields'] = '';
+				$item['headers'] = '';
+			}
+		}
+		unset($item);
 
 		$this->createReal($items);
 		$this->inherit($items);
@@ -515,7 +570,98 @@ class CItem extends CItemGeneral {
 
 		parent::checkInput($items, true);
 		self::validateInventoryLinks($items, true);
-		$this->validateDependentItems($items, API::Item());
+
+		$db_items = $this->get([
+			'output' => ['flags', 'type', 'master_itemid', 'authtype', 'allow_traps', 'retrieve_mode'],
+			'itemids' => zbx_objectValues($items, 'itemid'),
+			'editable' => true,
+			'preservekeys' => true
+		]);
+
+		$items = $this->extendFromObjects(zbx_toHash($items, 'itemid'), $db_items, ['flags', 'type', 'master_itemid']);
+
+		$this->validateDependentItems($items);
+
+		$defaults = DB::getDefaults('items');
+		$clean = [
+			ITEM_TYPE_HTTPAGENT => [
+				'url' => '',
+				'query_fields' => '',
+				'timeout' => $defaults['timeout'],
+				'status_codes' => $defaults['status_codes'],
+				'follow_redirects' => $defaults['follow_redirects'],
+				'request_method' => $defaults['request_method'],
+				'allow_traps' => $defaults['allow_traps'],
+				'post_type' => $defaults['post_type'],
+				'http_proxy' => '',
+				'headers' => '',
+				'retrieve_mode' => $defaults['retrieve_mode'],
+				'output_format' => $defaults['output_format'],
+				'ssl_key_password' => '',
+				'verify_peer' => $defaults['verify_peer'],
+				'verify_host' => $defaults['verify_host'],
+				'ssl_cert_file' => '',
+				'ssl_key_file' => '',
+				'posts' => ''
+			]
+		];
+
+		$json = new CJson();
+
+		foreach ($items as &$item) {
+			$type_change = ($item['type'] != $db_items[$item['itemid']]['type']);
+
+			if ($item['type'] != ITEM_TYPE_DEPENDENT && $db_items[$item['itemid']]['master_itemid'] != 0) {
+				$item['master_itemid'] = 0;
+			}
+
+			if ($type_change && $db_items[$item['itemid']]['type'] == ITEM_TYPE_HTTPAGENT) {
+				$item = array_merge($item, $clean[ITEM_TYPE_HTTPAGENT]);
+
+				if ($item['type'] != ITEM_TYPE_SSH) {
+					$item['authtype'] = $defaults['authtype'];
+					$item['username'] = '';
+					$item['password'] = '';
+				}
+
+				if ($item['type'] != ITEM_TYPE_TRAPPER) {
+					$item['trapper_hosts'] = '';
+				}
+			}
+
+			if ($item['type'] == ITEM_TYPE_HTTPAGENT) {
+				// Clean username and password on authtype change to HTTPTEST_AUTH_NONE.
+				if (array_key_exists('authtype', $item) && $item['authtype'] == HTTPTEST_AUTH_NONE
+						&& $item['authtype'] != $db_items[$item['itemid']]['authtype']) {
+					$item['username'] = '';
+					$item['password'] = '';
+				}
+
+				if (array_key_exists('allow_traps', $item) && $item['allow_traps'] == HTTPCHECK_ALLOW_TRAPS_OFF
+						&& $item['allow_traps'] != $db_items[$item['itemid']]['allow_traps']) {
+					$item['trapper_hosts'] = '';
+				}
+
+				if (array_key_exists('query_fields', $item) && is_array($item['query_fields'])) {
+					$item['query_fields'] = $item['query_fields'] ? $json->encode($item['query_fields']) : '';
+				}
+
+				if (array_key_exists('headers', $item) && is_array($item['headers'])) {
+					$item['headers'] = $this->headersArrayToString($item['headers']);
+				}
+
+				if (array_key_exists('request_method', $item) && $item['request_method'] == HTTPCHECK_REQUEST_HEAD
+						&& !array_key_exists('retrieve_mode', $item)
+						&& $db_items[$item['itemid']]['retrieve_mode'] != HTTPTEST_STEP_RETRIEVE_MODE_HEADERS) {
+					$item['retrieve_mode'] = HTTPTEST_STEP_RETRIEVE_MODE_HEADERS;
+				}
+			}
+			else {
+				$item['query_fields'] = '';
+				$item['headers'] = '';
+			}
+		}
+		unset($item);
 
 		$this->updateReal($items);
 		$this->inherit($items);
@@ -526,146 +672,54 @@ class CItem extends CItemGeneral {
 	/**
 	 * Delete items.
 	 *
-	 * @param array $itemIds
-	 * @param bool  $nopermissions
+	 * @param array $itemids
 	 *
 	 * @return array
 	 */
-	public function delete(array $itemIds, $nopermissions = false) {
-		if (!$itemIds) {
-			self::exception(ZBX_API_ERROR_PARAMETERS, _('Empty input parameter.'));
+	public function delete(array $itemids) {
+		$this->validateDelete($itemids, $db_items);
+
+		CItemManager::delete($itemids);
+
+		$this->addAuditBulk(AUDIT_ACTION_DELETE, AUDIT_RESOURCE_ITEM, $db_items);
+
+		return ['itemids' => $itemids];
+	}
+
+	/**
+	 * Validates the input parameters for the delete() method.
+	 *
+	 * @param array $itemids   [IN/OUT]
+	 * @param array $db_items  [OUT]
+	 *
+	 * @throws APIException if the input is invalid.
+	 */
+	private function validateDelete(array &$itemids, array &$db_items = null) {
+		$api_input_rules = ['type' => API_IDS, 'flags' => API_NOT_EMPTY, 'uniq' => true];
+		if (!CApiInputValidator::validate($api_input_rules, $itemids, '/', $error)) {
+			self::exception(ZBX_API_ERROR_PARAMETERS, $error);
 		}
 
-		$itemIds = array_keys(array_flip($itemIds));
-
-		$delItems = $this->get([
+		$db_items = $this->get([
 			'output' => ['itemid', 'name', 'templateid', 'flags'],
-			'itemids' => $itemIds,
+			'itemids' => $itemids,
 			'editable' => true,
 			'preservekeys' => true
 		]);
 
-		// TODO: remove $nopermissions hack
-		if (!$nopermissions) {
-			foreach ($itemIds as $itemId) {
-				if (!isset($delItems[$itemId])) {
-					self::exception(ZBX_API_ERROR_PERMISSIONS, _('No permissions to referred object or it does not exist!'));
-				}
-				$delItem = $delItems[$itemId];
+		foreach ($itemids as $itemid) {
+			if (!array_key_exists($itemid, $db_items)) {
+				self::exception(ZBX_API_ERROR_PERMISSIONS,
+					_('No permissions to referred object or it does not exist!')
+				);
+			}
 
-				if ($delItem['templateid'] != 0) {
-					self::exception(ZBX_API_ERROR_PARAMETERS, _('Cannot delete templated item.'));
-				}
+			$db_item = $db_items[$itemid];
+
+			if ($db_item['templateid'] != 0) {
+				self::exception(ZBX_API_ERROR_PARAMETERS, _('Cannot delete templated item.'));
 			}
 		}
-
-		// first delete child items
-		$parentItemIds = $itemIds;
-		$db_dependent_items = $delItems;
-		do {
-			$dbItems = DBselect('SELECT i.itemid FROM items i WHERE '.dbConditionInt('i.templateid',
-				$parentItemIds
-			));
-			$parentItemIds = [];
-			while ($dbItem = DBfetch($dbItems)) {
-				$parentItemIds[] = $dbItem['itemid'];
-				$itemIds[$dbItem['itemid']] = $dbItem['itemid'];
-				$db_dependent_items[$dbItem['itemid']] = $dbItem['itemid'];
-			}
-		} while ($parentItemIds);
-
-		$dependent_items = [];
-
-		while ($db_dependent_items) {
-			$db_dependent_items = $this->get([
-				'output' => ['itemid', 'name'],
-				'filter' => ['type' => ITEM_TYPE_DEPENDENT, 'master_itemid' => array_keys($db_dependent_items)],
-				'preservekeys' => true
-			]);
-			$db_dependent_items = array_diff_key($db_dependent_items, $dependent_items);
-			$dependent_items += $db_dependent_items;
-		};
-		$dependent_itemids = array_keys($dependent_items);
-		$itemIds += array_combine($dependent_itemids, $dependent_itemids);
-
-		// delete graphs, leave if graph still have item
-		$delGraphs = [];
-		$dbGraphs = DBselect(
-			'SELECT gi.graphid'.
-			' FROM graphs_items gi'.
-			' WHERE '.dbConditionInt('gi.itemid', $itemIds).
-				' AND NOT EXISTS ('.
-					'SELECT NULL'.
-					' FROM graphs_items gii'.
-					' WHERE gii.graphid=gi.graphid'.
-						' AND '.dbConditionInt('gii.itemid', $itemIds, true).
-				')'
-		);
-		while ($dbGraph = DBfetch($dbGraphs)) {
-			$delGraphs[$dbGraph['graphid']] = $dbGraph['graphid'];
-		}
-
-		if ($delGraphs) {
-			API::Graph()->delete($delGraphs, true);
-		}
-
-		// check if any graphs are referencing this item
-		$this->checkGraphReference($itemIds);
-
-		$triggers = API::Trigger()->get([
-			'output' => [],
-			'itemids' => $itemIds,
-			'nopermissions' => true,
-			'preservekeys' => true
-		]);
-		if ($triggers) {
-			API::Trigger()->delete(array_keys($triggers), true);
-		}
-
-		$triggerPrototypes = API::TriggerPrototype()->get([
-			'output' => [],
-			'itemids' => $itemIds,
-			'nopermissions' => true,
-			'preservekeys' => true
-		]);
-		if ($triggerPrototypes) {
-			API::TriggerPrototype()->delete(array_keys($triggerPrototypes), true);
-		}
-
-		DB::delete('screens_items', [
-			'resourceid' => $itemIds,
-			'resourcetype' => [SCREEN_RESOURCE_SIMPLE_GRAPH, SCREEN_RESOURCE_PLAIN_TEXT, SCREEN_RESOURCE_CLOCK]
-		]);
-
-		DB::delete('items', ['itemid' => $itemIds]);
-
-		DB::delete('profiles', [
-			'idx' => 'web.favorite.graphids',
-			'source' => 'itemid',
-			'value_id' => $itemIds
-		]);
-
-		$table_names = ['trends', 'trends_uint', 'history_text', 'history_log', 'history_uint', 'history_str',
-			'history', 'events'
-		];
-
-		$insert = [];
-
-		foreach ($itemIds as $itemId) {
-			foreach ($table_names as $table_name) {
-				$insert[] = [
-					'tablename' => $table_name,
-					'field' => 'itemid',
-					'value' => $itemId
-				];
-			}
-		}
-
-		DB::insertBatch('housekeeper', $insert);
-
-		$itemids = array_map('strval', array_values($itemIds));
-
-		return ['itemids' => $itemids];
 	}
 
 	public function syncTemplates($data) {
@@ -682,14 +736,32 @@ class CItem extends CItemGeneral {
 		$tpl_items = $this->get([
 			'output' => $output,
 			'selectApplications' => ['applicationid'],
-			'selectPreprocessing' => ['type', 'params'],
+			'selectPreprocessing' => ['type', 'params', 'error_handler', 'error_handler_params'],
 			'hostids' => $data['templateids'],
 			'filter' => ['flags' => ZBX_FLAG_DISCOVERY_NORMAL],
 			'preservekeys' => true
 		]);
 
+		$json = new CJson();
+
 		foreach ($tpl_items as &$tpl_item) {
 			$tpl_item['applications'] = zbx_objectValues($tpl_item['applications'], 'applicationid');
+
+			if ($tpl_item['type'] == ITEM_TYPE_HTTPAGENT) {
+				if (array_key_exists('query_fields', $tpl_item) && is_array($tpl_item['query_fields'])) {
+					$tpl_item['query_fields'] = $tpl_item['query_fields']
+						? $json->encode($tpl_item['query_fields'])
+						: '';
+				}
+
+				if (array_key_exists('headers', $tpl_item) && is_array($tpl_item['headers'])) {
+					$tpl_item['headers'] = $this->headersArrayToString($tpl_item['headers']);
+				}
+			}
+			else {
+				$tpl_item['query_fields'] = '';
+				$tpl_item['headers'] = '';
+			}
 		}
 		unset($tpl_item);
 
@@ -724,69 +796,6 @@ class CItem extends CItemGeneral {
 				_s('Incorrect value for field "%1$s": %2$s.', 'trends', $error)
 			);
 		}
-
-		$this->validateItemPreprocessing($item, $method);
-	}
-
-	protected function inherit(array $items, array $hostids = null) {
-		if (!$items) {
-			return;
-		}
-
-		// Prepare the child items.
-		$new_items = $this->prepareInheritedItems($items, $hostids);
-		if (!$new_items) {
-			return;
-		}
-
-		$ins_items = [];
-		$upd_items = [];
-		foreach ($new_items as $new_item) {
-			if (array_key_exists('itemid', $new_item)) {
-				$upd_items[] = $new_item;
-			}
-			else {
-				$ins_items[] = $new_item;
-			}
-		}
-
-		// Save the new items.
-		if ($ins_items) {
-			self::validateInventoryLinks($ins_items, false); // false means 'create'
-			$this->createReal($ins_items);
-		}
-
-		if ($upd_items) {
-			self::validateInventoryLinks($upd_items, true); // true means 'update'
-			$this->updateReal($upd_items);
-		}
-
-		$new_items = array_merge($upd_items, $ins_items);
-
-		// Update master_itemid for inserted or updated inherited dependent items.
-		$this->inheritDependentItems($new_items);
-
-		// Inheriting items from the templates.
-		$tpl_items = DBselect(
-			'SELECT i.itemid'.
-			' FROM items i,hosts h'.
-			' WHERE i.hostid=h.hostid'.
-				' AND '.dbConditionInt('i.itemid', zbx_objectValues($new_items, 'itemid')).
-				' AND '.dbConditionInt('h.status', [HOST_STATUS_TEMPLATE])
-		);
-
-		$tpl_itemids = [];
-		while ($tpl_item = DBfetch($tpl_items)) {
-			$tpl_itemids[$tpl_item['itemid']] = true;
-		}
-
-		foreach ($new_items as $index => $new_item) {
-			if (!array_key_exists($new_item['itemid'], $tpl_itemids)) {
-				unset($new_items[$index]);
-			}
-		}
-
-		$this->inherit($new_items);
 	}
 
 	/**
@@ -1134,6 +1143,8 @@ class CItem extends CItemGeneral {
 			foreach ($result as &$item) {
 				$lastHistory = isset($history[$item['itemid']][0]) ? $history[$item['itemid']][0] : null;
 				$prevHistory = isset($history[$item['itemid']][1]) ? $history[$item['itemid']][1] : null;
+				$no_value = in_array($item['value_type'],
+						[ITEM_VALUE_TYPE_STR, ITEM_VALUE_TYPE_LOG, ITEM_VALUE_TYPE_TEXT]) ? '' : '0';
 
 				if (isset($requestedOutput['lastclock'])) {
 					$item['lastclock'] = $lastHistory ? $lastHistory['clock'] : '0';
@@ -1142,38 +1153,15 @@ class CItem extends CItemGeneral {
 					$item['lastns'] = $lastHistory ? $lastHistory['ns'] : '0';
 				}
 				if (isset($requestedOutput['lastvalue'])) {
-					$item['lastvalue'] = $lastHistory ? $lastHistory['value'] : '0';
+					$item['lastvalue'] = $lastHistory ? $lastHistory['value'] : $no_value;
 				}
 				if (isset($requestedOutput['prevvalue'])) {
-					$item['prevvalue'] = $prevHistory ? $prevHistory['value'] : '0';
+					$item['prevvalue'] = $prevHistory ? $prevHistory['value'] : $no_value;
 				}
 			}
 			unset($item);
 		}
 
-		if ($options['selectPreprocessing'] !== null && $options['selectPreprocessing'] != API_OUTPUT_COUNT) {
-			$db_item_preproc = API::getApiService()->select('item_preproc', [
-				'output' => $this->outputExtend($options['selectPreprocessing'], ['itemid', 'step']),
-				'filter' => ['itemid' => array_keys($result)],
-			]);
-
-			CArrayHelper::sort($db_item_preproc, ['step']);
-
-			foreach ($result as &$item) {
-				$item['preprocessing'] = [];
-			}
-			unset($item);
-
-			foreach ($db_item_preproc as $step) {
-				$itemid = $step['itemid'];
-				unset($step['item_preprocid'], $step['itemid'], $step['step']);
-
-				if (array_key_exists($itemid, $result)) {
-					$result[$itemid]['preprocessing'][] = $step;
-				}
-			}
-		}
-
 		return $result;
 	}
 
