@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -72,6 +72,8 @@ class CTemplate extends CHostGeneral {
 			'editable'					=> false,
 			'nopermissions'				=> null,
 			// filter
+			'evaltype'					=> TAG_EVAL_TYPE_AND_OR,
+			'tags'						=> null,
 			'filter'					=> null,
 			'search'					=> '',
 			'searchByAny'				=> null,
@@ -92,6 +94,7 @@ class CTemplate extends CHostGeneral {
 			'selectMacros'				=> null,
 			'selectScreens'				=> null,
 			'selectHttpTests'			=> null,
+			'selectTags'				=> null,
 			'countOutput'				=> false,
 			'groupCount'				=> false,
 			'preservekeys'				=> false,
@@ -236,6 +239,13 @@ class CTemplate extends CHostGeneral {
 			$sqlParts['where'][] = 'EXISTS (SELECT ht.httptestid FROM httptest ht WHERE ht.hostid=h.hostid)';
 		}
 
+		// tags
+		if ($options['tags'] !== null && $options['tags']) {
+			$sqlParts['where'][] = CApiTagHelper::addWhereCondition($options['tags'], $options['evaltype'], 'h',
+				'host_tag', 'hostid'
+			);
+		}
+
 		// filter
 		if (is_array($options['filter'])) {
 			$this->dbFilter('hosts h', $options, $sqlParts);
@@ -306,6 +316,7 @@ class CTemplate extends CHostGeneral {
 			$templates[$key]['groups'] = zbx_toArray($template['groups']);
 		}
 
+		$ins_tags = [];
 		foreach ($templates as $template) {
 			// if visible name is not given or empty it should be set to host name
 			if ((!isset($template['name']) || zbx_empty(trim($template['name']))) && isset($template['host'])) {
@@ -323,6 +334,12 @@ class CTemplate extends CHostGeneral {
 
 			$templateIds[] = $templateId;
 
+			if (array_key_exists('tags', $template)) {
+				foreach ($template['tags'] as $tag) {
+					$ins_tags[] = ['hostid' => $templateId] + $tag;
+				}
+			}
+
 			foreach ($template['groups'] as $group) {
 				$hostGroupId = get_dbid('hosts_groups', 'hostgroupid');
 
@@ -350,6 +367,10 @@ class CTemplate extends CHostGeneral {
 			}
 		}
 
+		if ($ins_tags) {
+			DB::insert('host_tag', $ins_tags);
+		}
+
 		return ['templateids' => $templateIds];
 	}
 
@@ -357,6 +378,8 @@ class CTemplate extends CHostGeneral {
 	 * Validate create template.
 	 *
 	 * @param array $templates
+	 *
+	 * @throws APIException if the input is invalid.
 	 */
 	protected function validateCreate(array $templates) {
 		$groupIds = [];
@@ -462,6 +485,11 @@ class CTemplate extends CHostGeneral {
 					));
 				}
 			}
+
+			// Validate tags.
+			if (array_key_exists('tags', $template)) {
+				$this->validateTags($template);
+			}
 		}
 	}
 
@@ -508,6 +536,8 @@ class CTemplate extends CHostGeneral {
 			}
 		}
 
+		$this->updateTags($templates, 'templateid');
+
 		return ['templateids' => zbx_objectValues($templates, 'templateid')];
 	}
 
@@ -515,6 +545,8 @@ class CTemplate extends CHostGeneral {
 	 * Validate update template.
 	 *
 	 * @param array $templates
+	 *
+	 * @throws APIException if the input is invalid.
 	 */
 	protected function validateUpdate(array $templates) {
 		$dbTemplates = $this->get([
@@ -533,6 +565,11 @@ class CTemplate extends CHostGeneral {
 			if (array_key_exists('auto_compress', $template)) {
 				self::exception(ZBX_API_ERROR_PARAMETERS, _('Incorrect input parameters.'));
 			}
+
+			// Validate tags.
+			if (array_key_exists('tags', $template)) {
+				$this->validateTags($template);
+			}
 		}
 	}
 
@@ -565,25 +602,25 @@ class CTemplate extends CHostGeneral {
 		API::Template()->unlink($templateids, null, true);
 
 		// delete the discovery rules first
-		$delRules = API::DiscoveryRule()->get([
-			'output' => ['itemid'],
+		$del_rules = API::DiscoveryRule()->get([
+			'output' => [],
 			'hostids' => $templateids,
 			'nopermissions' => true,
 			'preservekeys' => true
 		]);
-		if ($delRules) {
-			API::DiscoveryRule()->delete(array_keys($delRules), true);
+		if ($del_rules) {
+			API::DiscoveryRule()->delete(array_keys($del_rules), true);
 		}
 
 		// delete the items
-		$delItems = API::Item()->get([
+		$del_items = API::Item()->get([
+			'output' => [],
 			'templateids' => $templateids,
-			'output' => ['itemid'],
 			'nopermissions' => true,
 			'preservekeys' => true
 		]);
-		if ($delItems) {
-			API::Item()->delete(array_keys($delItems), true);
+		if ($del_items) {
+			CItemManager::delete(array_keys($del_items));
 		}
 
 		// delete host from maps
@@ -754,8 +791,6 @@ class CTemplate extends CHostGeneral {
 				'message' => _('Cannot update templates on discovered host "%1$s".')
 			]));
 
-			$this->validateDependentItemsLinkage($hostIds, $templateIds);
-
 			$this->link($templateIds, $hostIds);
 		}
 
