@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2017 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -195,12 +195,13 @@ abstract class CTriggerGeneral extends CApiService {
 				'filter' => ['triggerid' => $db_trigger['triggerid']]
 			]);
 
-			$this->updateReal([$trigger], [$db_trigger]);
+			$this->updateReal([$trigger], [$db_trigger], true);
 		}
 		else {
 			$_db_triggers = $this->get([
 				'output' => ['url', 'status', 'priority', 'comments', 'type', 'correlation_mode', 'correlation_tag'],
-				'triggerids' => [$triggerid]
+				'triggerids' => [$triggerid],
+				'nopermissions' => true
 			]);
 
 			foreach ($_db_triggers[0] as $field_name => $value) {
@@ -210,7 +211,7 @@ abstract class CTriggerGeneral extends CApiService {
 			}
 
 			$triggers = [$trigger];
-			$this->createReal($triggers);
+			$this->createReal($triggers, true);
 			$trigger = $triggers[0];
 		}
 
@@ -391,6 +392,13 @@ abstract class CTriggerGeneral extends CApiService {
 				'filter' => ['triggerid' => $triggerids],
 				'preservekeys' => true
 			]);
+
+			// Rename column 'name' to 'function'.
+			$function = reset($functions);
+			if ($function && array_key_exists('name', $function)) {
+				$functions = CArrayHelper::renameObjectsKeys($functions, ['name' => 'function']);
+			}
+
 			$relationMap = $this->createRelationMap($functions, 'triggerid', 'functionid');
 
 			$functions = $this->unsetExtraFields($functions, ['triggerid', 'functionid'], $options['selectFunctions']);
@@ -858,10 +866,12 @@ abstract class CTriggerGeneral extends CApiService {
 	 * @param string $triggers[]['tags'][]['value']     [IN]
 	 * @param int    $triggers[]['correlation_mode']    [IN] (optional)
 	 * @param string $triggers[]['correlation_tag']     [IN] (optional)
+	 * @param bool   $inherited                         [IN] (optional)  If set to true, trigger will be created for
+	 *                                                                   non-editable host/template.
 	 *
 	 * @throws APIException
 	 */
-	protected function createReal(array &$triggers) {
+	protected function createReal(array &$triggers, $inherited = false) {
 		if (!$triggers) {
 			return;
 		}
@@ -885,7 +895,7 @@ abstract class CTriggerGeneral extends CApiService {
 		$new_functions = [];
 		$triggers_functions = [];
 		$new_tags = [];
-		$this->implode_expressions($new_triggers, null, $triggers_functions);
+		$this->implode_expressions($new_triggers, null, $triggers_functions, $inherited);
 
 		$triggerid = DB::reserveIds('triggers', count($new_triggers));
 
@@ -966,10 +976,13 @@ abstract class CTriggerGeneral extends CApiService {
 	 * @param string $db_triggers[<tnum>]['tags'][]['value']         [IN]
 	 * @param int    $db_triggers[<tnum>]['correlation_mode']        [IN]
 	 * @param string $db_triggers[<tnum>]['correlation_tag']         [IN]
+	 * @param bool   $inherited                                      [IN] (optional)  If set to true, trigger will be
+	 *                                                                                created for non-editable
+	 *                                                                                host/template.
 	 *
 	 * @throws APIException
 	 */
-	protected function updateReal(array $triggers, array $db_triggers) {
+	protected function updateReal(array $triggers, array $db_triggers, $inherited = false) {
 		if (!$triggers) {
 			return;
 		}
@@ -996,7 +1009,7 @@ abstract class CTriggerGeneral extends CApiService {
 		$new_tags = [];
 		$del_triggertagids = [];
 		$save_triggers = $triggers;
-		$this->implode_expressions($triggers, $db_triggers, $triggers_functions);
+		$this->implode_expressions($triggers, $db_triggers, $triggers_functions, $inherited);
 
 		if ($class === 'CTrigger') {
 			// The list of the triggers with changed priority.
@@ -1107,7 +1120,7 @@ abstract class CTriggerGeneral extends CApiService {
 		}
 
 		if ($class === 'CTrigger' && $changed_priority_triggerids
-				&& $this->usedInItServices($changed_priority_triggerids)) {
+				&& CTriggerManager::usedInItServices($changed_priority_triggerids)) {
 			updateItServices();
 		}
 
@@ -1141,12 +1154,16 @@ abstract class CTriggerGeneral extends CApiService {
 	 * @param string     $triggers_functions[<tnum>][]['functionid'] [OUT]
 	 * @param null       $triggers_functions[<tnum>][]['triggerid']  [OUT] must be initialized before insertion into DB
 	 * @param string     $triggers_functions[<tnum>][]['itemid']     [OUT]
-	 * @param string     $triggers_functions[<tnum>][]['function']   [OUT]
+	 * @param string     $triggers_functions[<tnum>][]['name']       [OUT]
 	 * @param string     $triggers_functions[<tnum>][]['parameter']  [OUT]
+	 * @param bool       $inherited                                  [IN] (optional)  If set to true, triggers will be
+	 *                                                                                created for non-editable
+	 *                                                                                hosts/templates.
 	 *
 	 * @throws APIException if error occurred
 	 */
-	function implode_expressions(array &$triggers, array $db_triggers = null, array &$triggers_functions) {
+	function implode_expressions(array &$triggers, array $db_triggers = null, array &$triggers_functions,
+			$inherited = false) {
 		$class = get_class($this);
 
 		switch ($class) {
@@ -1229,18 +1246,20 @@ abstract class CTriggerGeneral extends CApiService {
 			return;
 		}
 
+		$permission_check = $inherited
+			? ['nopermissions' => true]
+			: ['editable' => true];
+
 		$_db_hosts = API::Host()->get([
 			'output' => ['hostid', 'host', 'status'],
-			'filter' => ['host' => array_keys($hosts_keys)],
-			'editable' => true
-		]);
+			'filter' => ['host' => array_keys($hosts_keys)]
+		] + $permission_check);
 
 		if (count($hosts_keys) != count($_db_hosts)) {
 			$_db_templates = API::Template()->get([
 				'output' => ['templateid', 'host', 'status'],
-				'filter' => ['host' => array_keys($hosts_keys)],
-				'editable' => true
-			]);
+				'filter' => ['host' => array_keys($hosts_keys)]
+			] + $permission_check);
 
 			foreach ($_db_templates as &$_db_template) {
 				$_db_template['hostid'] = $_db_template['templateid'];
@@ -1378,7 +1397,7 @@ abstract class CTriggerGeneral extends CApiService {
 						'functionid' => null,
 						'triggerid' => null,
 						'itemid' => $key['itemid'],
-						'function' => $exprPart['functionName'],
+						'name' => $exprPart['functionName'],
 						'parameter' => $exprPart['functionParam']
 					];
 					$functions_num++;
