@@ -4,9 +4,11 @@
 		jQuery('#user_medias_' + index).remove();
 		// hidden variables
 		jQuery('#user_medias_' + index + '_mediaid').remove();
+		jQuery('#user_medias_' + index + '_mediatype').remove();
 		jQuery('#user_medias_' + index + '_mediatypeid').remove();
 		jQuery('#user_medias_' + index + '_period').remove();
 		jQuery('#user_medias_' + index + '_sendto').remove();
+		removeVarsBySelector(null, 'input[id^="user_medias_' + index + '_sendto_"]');
 		jQuery('#user_medias_' + index + '_severity').remove();
 		jQuery('#user_medias_' + index + '_active').remove();
 		jQuery('#user_medias_' + index + '_description').remove();
@@ -31,8 +33,9 @@
 
 		<?php if ($this->data['is_profile']): ?>
 			jQuery('#messages_enabled').on('change', function() {
-				jQuery('#messagingTab input, #messagingTab button, #messagingTab select').prop('disabled', !this.checked);
-				jQuery('#messages_enabled').prop('disabled', false);
+				jQuery('#messagingTab input, #messagingTab button, #messagingTab select')
+					.not('[name="messages[enabled]"]')
+					.prop('disabled', !this.checked);
 			});
 
 			jQuery('#messages_enabled').trigger('change');
